@@ -13,6 +13,10 @@
 #include "cpl_string.h"
 #include "gdal_frmts.h"
 #include "gdal_pam.h"
+#include "gdal_driver.h"
+#include "gdal_drivermanager.h"
+#include "gdal_openinfo.h"
+#include "gdal_cpp_functions.h"
 #include "ogr_spatialref.h"
 
 #include <cstdlib>
@@ -34,7 +38,7 @@ class HF2Dataset final : public GDALPamDataset
     friend class HF2RasterBand;
 
     VSILFILE *fp;
-    double adfGeoTransform[6];
+    GDALGeoTransform m_gt{};
     OGRSpatialReference m_oSRS{};
     vsi_l_offset *panBlockOffset;  // tile 0 is a the bottom left
 
@@ -44,9 +48,9 @@ class HF2Dataset final : public GDALPamDataset
 
   public:
     HF2Dataset();
-    virtual ~HF2Dataset();
+    ~HF2Dataset() override;
 
-    virtual CPLErr GetGeoTransform(double *) override;
+    CPLErr GetGeoTransform(GDALGeoTransform &gt) const override;
     const OGRSpatialReference *GetSpatialRef() const override;
 
     static GDALDataset *Open(GDALOpenInfo *);
@@ -73,9 +77,9 @@ class HF2RasterBand final : public GDALPamRasterBand
 
   public:
     HF2RasterBand(HF2Dataset *, int, GDALDataType);
-    virtual ~HF2RasterBand();
+    ~HF2RasterBand() override;
 
-    virtual CPLErr IReadBlock(int, int, void *) override;
+    CPLErr IReadBlock(int, int, void *) override;
 };
 
 /************************************************************************/
@@ -110,7 +114,7 @@ HF2RasterBand::~HF2RasterBand()
 CPLErr HF2RasterBand::IReadBlock(int nBlockXOff, int nLineYOff, void *pImage)
 
 {
-    HF2Dataset *poGDS = (HF2Dataset *)poDS;
+    HF2Dataset *poGDS = cpl::down_cast<HF2Dataset *>(poDS);
 
     const int nXBlocks = DIV_ROUND_UP(nRasterXSize, poGDS->nTileSize);
 
@@ -257,12 +261,6 @@ HF2Dataset::HF2Dataset()
       bHasLoaderBlockMap(FALSE)
 {
     m_oSRS.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
-    adfGeoTransform[0] = 0;
-    adfGeoTransform[1] = 1;
-    adfGeoTransform[2] = 0;
-    adfGeoTransform[3] = 0;
-    adfGeoTransform[4] = 0;
-    adfGeoTransform[5] = 1;
 }
 
 /************************************************************************/
@@ -289,8 +287,8 @@ int HF2Dataset::LoadBlockMap()
 
     bHasLoaderBlockMap = TRUE;
 
-    const int nXBlocks = (nRasterXSize + nTileSize - 1) / nTileSize;
-    const int nYBlocks = (nRasterYSize + nTileSize - 1) / nTileSize;
+    const int nXBlocks = DIV_ROUND_UP(nRasterXSize, nTileSize);
+    const int nYBlocks = DIV_ROUND_UP(nRasterYSize, nTileSize);
     if (nXBlocks * nYBlocks > 1000000)
     {
         vsi_l_offset nCurOff = VSIFTellL(fp);
@@ -477,8 +475,8 @@ GDALDataset *HF2Dataset::Open(GDALOpenInfo *poOpenInfo)
     {
         return nullptr;
     }
-    const int nXBlocks = (nXSize + nTileSize - 1) / nTileSize;
-    const int nYBlocks = (nYSize + nTileSize - 1) / nTileSize;
+    const int nXBlocks = DIV_ROUND_UP(nXSize, nTileSize);
+    const int nYBlocks = DIV_ROUND_UP(nYSize, nTileSize);
     if (nXBlocks > INT_MAX / nYBlocks)
     {
         return nullptr;
@@ -598,15 +596,15 @@ GDALDataset *HF2Dataset::Open(GDALOpenInfo *poOpenInfo)
              nTileSize);
     if (bHasExtent)
     {
-        poDS->adfGeoTransform[0] = dfMinX;
-        poDS->adfGeoTransform[3] = dfMaxY;
-        poDS->adfGeoTransform[1] = (dfMaxX - dfMinX) / nXSize;
-        poDS->adfGeoTransform[5] = -(dfMaxY - dfMinY) / nYSize;
+        poDS->m_gt[0] = dfMinX;
+        poDS->m_gt[3] = dfMaxY;
+        poDS->m_gt[1] = (dfMaxX - dfMinX) / nXSize;
+        poDS->m_gt[5] = -(dfMaxY - dfMinY) / nYSize;
     }
     else
     {
-        poDS->adfGeoTransform[1] = fHorizScale;
-        poDS->adfGeoTransform[5] = fHorizScale;
+        poDS->m_gt[1] = fHorizScale;
+        poDS->m_gt[5] = fHorizScale;
     }
 
     if (bHasEPSGCode)
@@ -681,10 +679,10 @@ GDALDataset *HF2Dataset::Open(GDALOpenInfo *poOpenInfo)
 /*                          GetGeoTransform()                           */
 /************************************************************************/
 
-CPLErr HF2Dataset::GetGeoTransform(double *padfTransform)
+CPLErr HF2Dataset::GetGeoTransform(GDALGeoTransform &gt) const
 
 {
-    memcpy(padfTransform, adfGeoTransform, 6 * sizeof(double));
+    gt = m_gt;
 
     return CE_None;
 }
@@ -752,13 +750,11 @@ GDALDataset *HF2Dataset::CreateCopy(const char *pszFilename,
 
     const int nXSize = poSrcDS->GetRasterXSize();
     const int nYSize = poSrcDS->GetRasterYSize();
-    double adfGeoTransform[6];
-    poSrcDS->GetGeoTransform(adfGeoTransform);
-    int bHasGeoTransform =
-        !(adfGeoTransform[0] == 0 && adfGeoTransform[1] == 1 &&
-          adfGeoTransform[2] == 0 && adfGeoTransform[3] == 0 &&
-          adfGeoTransform[4] == 0 && adfGeoTransform[5] == 1);
-    if (adfGeoTransform[2] != 0 || adfGeoTransform[4] != 0)
+    GDALGeoTransform gt;
+    const bool bHasGeoTransform = poSrcDS->GetGeoTransform(gt) == CE_None &&
+                                  !(gt[0] == 0 && gt[1] == 1 && gt[2] == 0 &&
+                                    gt[3] == 0 && gt[4] == 0 && gt[5] == 1);
+    if (gt[2] != 0 || gt[4] != 0)
     {
         CPLError(CE_Failure, CPLE_NotSupported,
                  "HF2 driver does not support CreateCopy() from skewed or "
@@ -901,8 +897,7 @@ GDALDataset *HF2Dataset::CreateCopy(const char *pszFilename,
     WriteInt(fp, nYSize);
     WriteShort(fp, (GInt16)nTileSize);
     WriteFloat(fp, fVertPres);
-    const float fHorizScale =
-        (float)((fabs(adfGeoTransform[1]) + fabs(adfGeoTransform[5])) / 2);
+    const float fHorizScale = (float)((fabs(gt[1]) + fabs(gt[5])) / 2);
     WriteFloat(fp, fHorizScale);
     WriteInt(fp, nExtendedHeaderLen);
 
@@ -919,10 +914,10 @@ GDALDataset *HF2Dataset::CreateCopy(const char *pszFilename,
         VSIFWriteL(szBlockName, 16, 1, fp);
         WriteInt(fp, 34);
         WriteShort(fp, (GInt16)nExtentUnits);
-        WriteDouble(fp, adfGeoTransform[0]);
-        WriteDouble(fp, adfGeoTransform[0] + nXSize * adfGeoTransform[1]);
-        WriteDouble(fp, adfGeoTransform[3] + nYSize * adfGeoTransform[5]);
-        WriteDouble(fp, adfGeoTransform[3]);
+        WriteDouble(fp, gt[0]);
+        WriteDouble(fp, gt[0] + nXSize * gt[1]);
+        WriteDouble(fp, gt[3] + nYSize * gt[5]);
+        WriteDouble(fp, gt[3]);
     }
     if (nUTMZone != 0)
     {
@@ -955,8 +950,8 @@ GDALDataset *HF2Dataset::CreateCopy(const char *pszFilename,
     /* -------------------------------------------------------------------- */
     /*      Copy imagery                                                    */
     /* -------------------------------------------------------------------- */
-    const int nXBlocks = (nXSize + nTileSize - 1) / nTileSize;
-    const int nYBlocks = (nYSize + nTileSize - 1) / nTileSize;
+    const int nXBlocks = DIV_ROUND_UP(nXSize, nTileSize);
+    const int nYBlocks = DIV_ROUND_UP(nYSize, nTileSize);
 
     void *pTileBuffer = VSI_MALLOC3_VERBOSE(nTileSize, nTileSize,
                                             GDALGetDataTypeSizeBytes(eReqDT));
@@ -1170,7 +1165,7 @@ GDALDataset *HF2Dataset::CreateCopy(const char *pszFilename,
         return nullptr;
 
     GDALOpenInfo oOpenInfo(osFilename.c_str(), GA_ReadOnly);
-    HF2Dataset *poDS = reinterpret_cast<HF2Dataset *>(Open(&oOpenInfo));
+    HF2Dataset *poDS = cpl::down_cast<HF2Dataset *>(Open(&oOpenInfo));
 
     if (poDS)
         poDS->CloneInfo(poSrcDS, GCIF_PAM_DEFAULT);
